@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: lid.c,v 1.20 2006/03/06 03:02:52 mitr Exp $"
+#ident "$Id: lid.c,v 1.21 2007/02/16 20:33:01 mitr Exp $"
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -26,6 +26,7 @@
 #include <locale.h>
 #include <popt.h>
 #include <pwd.h>
+#include <stdint.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include "../lib/user.h"
@@ -65,14 +66,13 @@ do_id (struct lu_context *ctx, const char *name, int nameonly,
 
 				attrs = lu_ent_get(ent, id_attribute);
 				if (attrs != NULL) {
-					char *tmp;
+					id_t id;
 
 					value = g_value_array_get_nth(attrs,
 								      0);
-					tmp = lu_value_strdup(value);
-					g_print(" %s(%s=%s)\n", found,
-						id_descr, tmp);
-					g_free(tmp);
+					id = lu_value_get_id(value);
+					g_print(" %s(%s=%jd)\n", found,
+						id_descr, (intmax_t)id);
 				} else
 					g_print(" %s\n", found);
 			} else {
