@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: lnewusers.c,v 1.23 2006/05/01 04:51:21 mitr Exp $"
+#ident "$Id: lnewusers.c,v 1.25 2007/02/17 12:33:51 mitr Exp $"
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -108,14 +108,12 @@ main(int argc, const char **argv)
 		GValue *value, val;
 
 		/* Strip off the end-of-line terminators. */
-		if (strchr(buf, '\r')) {
-			p = strchr(buf, '\r');
+		p = strchr(buf, '\r');
+		if (p != NULL)
 			*p = '\0';
-		}
-		if (strchr(buf, '\n')) {
-			p = strchr(buf, '\n');
+		p = strchr(buf, '\n');
+		if (p != NULL)
 			*p = '\0';
-		}
 
 		/* Make sure the line splits into *exactly* seven fields. */
 		fields = g_strsplit(buf, ":", 7);
@@ -259,10 +257,9 @@ main(int argc, const char **argv)
 				value = g_value_array_get_nth(values,
 							      0);
 				homedir = g_strdup(g_value_get_string(value));
-			} else {
-				homedir = g_strdup_printf("/home/%s",
-							  fields[0]);
-			}
+			} else
+				homedir = g_strconcat("/home/", fields[0],
+						      (const gchar *)NULL);
 		}
 		if (strlen(fields[6]) > 0) {
 			g_value_set_string(&val, fields[6]);
