@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: entity.c,v 1.31 2005/10/09 05:15:47 mitr Exp $"
+#ident "$Id: entity.c,v 1.34 2007/02/15 15:46:59 mitr Exp $"
 
 #ifdef HAVE_CONFIG_H
 #include "../config.h"
@@ -90,14 +90,44 @@ lu_ent_free(struct lu_ent *ent)
 	g_free(ent);
 }
 
+/* Dump a set of attributes */
+static void
+lu_ent_dump_attributes(GArray *attrs, FILE *fp)
+{
+	size_t i;
+
+	for (i = 0; i < attrs->len; i++) {
+		struct lu_attribute *attribute;
+		size_t j;
+
+		attribute = &g_array_index(attrs, struct lu_attribute, i);
+		for (j = 0; j < attribute->values->n_values; j++) {
+			GValue *value;
+
+			value = g_value_array_get_nth(attribute->values, j);
+			fprintf(fp, " %s = ",
+				g_quark_to_string(attribute->name));
+			if (G_VALUE_HOLDS_STRING(value))
+				fprintf(fp, "`%s'\n",
+					g_value_get_string(value));
+			else if (G_VALUE_HOLDS_LONG(value))
+				fprintf(fp, "%ld\n", g_value_get_long(value));
+			else if (G_VALUE_HOLDS_INT64(value))
+				fprintf(fp, "%lld\n",
+					(long long)g_value_get_int64(value));
+			else
+				fprintf(fp, "???\n");
+		}
+	}
+}
+
 /* Dump the contents of an entity structure.  This is primarily for
  * debugging. */
 void
 lu_ent_dump(struct lu_ent *ent, FILE *fp)
 {
-	size_t i, j;
-	struct lu_attribute *attribute;
-	GValue *value;
+	size_t i;
+
 	g_return_if_fail(ent != NULL);
 	fprintf(fp, "dump of struct lu_ent at %p:\n", ent);
 	fprintf(fp, " magic = %08x\n", ent->magic);
@@ -120,81 +150,32 @@ lu_ent_dump(struct lu_ent *ent, FILE *fp)
 	/* Print the module list. */
 	fprintf(fp, " modules = (");
 	for (i = 0; i < ent->modules->n_values; i++) {
+		GValue *value;
+
 		value = g_value_array_get_nth(ent->modules, i);
-		if (i > 0) {
+		if (i > 0)
 			fprintf(fp, ", ");
-		}
 		if (G_VALUE_HOLDS_STRING(value))
 			fprintf(fp, "`%s'", g_value_get_string(value));
-		else if (G_VALUE_HOLDS_LONG(value))
-			fprintf(fp, "%ld", g_value_get_long(value));
-		else if (G_VALUE_HOLDS_INT64(value))
-			fprintf(fp, "%lld",
-				(long long)g_value_get_int64(value));
 		else
 			fprintf(fp, "?");
 	}
 	fprintf(fp, ")\n");
 	/* Print the current data values. */
-	for (i = 0; i < ent->current->len; i++) {
-		attribute = &g_array_index(ent->current,
-					   struct lu_attribute,
-					   i);
-		for (j = 0; j < attribute->values->n_values; j++) {
-			value = g_value_array_get_nth(attribute->values, j);
-			if (G_VALUE_HOLDS_STRING(value))
-				fprintf(fp, " %s = `%s'\n",
-					g_quark_to_string(attribute->name),
-					g_value_get_string(value));
-			else if (G_VALUE_HOLDS_LONG(value))
-				fprintf(fp, " %s = %ld\n",
-					g_quark_to_string(attribute->name),
-					g_value_get_long(value));
-
-			else if (G_VALUE_HOLDS_INT64(value))
-				fprintf(fp, " %s = %lld\n",
-					g_quark_to_string(attribute->name),
-					(long long)g_value_get_int64(value));
-
-		}
-	}
+	lu_ent_dump_attributes(ent->current, fp);
 	fprintf(fp, "\n");
-	for (i = 0; i < ent->pending->len; i++) {
-		attribute = &g_array_index(ent->pending,
-					   struct lu_attribute,
-					   i);
-		for (j = 0; j < attribute->values->n_values; j++) {
-			value = g_value_array_get_nth(attribute->values, j);
-			if (G_VALUE_HOLDS_STRING(value))
-				fprintf(fp, " %s = `%s'\n",
-					g_quark_to_string(attribute->name),
-					g_value_get_string(value));
-			else if (G_VALUE_HOLDS_LONG(value))
-				fprintf(fp, " %s = %ld\n",
-					g_quark_to_string(attribute->name),
-					g_value_get_long(value));
-			else if (G_VALUE_HOLDS_INT64(value))
-				fprintf(fp, " %s = %lld\n",
-					g_quark_to_string(attribute->name),
-					(long long)g_value_get_int64(value));
-		}
-	}
+	lu_ent_dump_attributes(ent->pending, fp);
 }
 
 /* Add a module to the list of modules kept for this entity. */
 void
 lu_ent_add_module(struct lu_ent *ent, const char *source)
 {
-	GValue value;
 	size_t i;
+
 	g_return_if_fail(ent != NULL);
 	g_return_if_fail(ent->magic == LU_ENT_MAGIC);
 	g_return_if_fail(ent->modules != NULL);
-	/* Initialize a value with the string. */
-	memset(&value, 0, sizeof(value));
-	g_value_init(&value, G_TYPE_STRING);
-	g_value_set_string(&value, source);
-	/* Now scan the array for the value. */
 	for (i = 0; i < ent->modules->n_values; i++) {
 		GValue *val;
 
@@ -202,17 +183,21 @@ lu_ent_add_module(struct lu_ent *ent, const char *source)
 		/* We only add strings, so there had better be only strings
 		 * in this list, otherwise someone is messing with us. */
 		g_assert(G_VALUE_HOLDS_STRING(val));
-		if (strcmp(g_value_get_string(val),
-			   g_value_get_string(&value)) == 0) {
+		if (strcmp(g_value_get_string(val), source) == 0)
 			break;
-		}
 	}
 	/* If we fell of the end of the array, then the new value is not
 	 * in there, so we should add it. */
 	if (i >= ent->modules->n_values) {
+		GValue value;
+
+		/* Initialize a value with the string. */
+		memset(&value, 0, sizeof(value));
+		g_value_init(&value, G_TYPE_STRING);
+		g_value_set_string(&value, source);
 		g_value_array_append(ent->modules, &value);
+		g_value_unset(&value);
 	}
-	g_value_unset(&value);
 }
 
 /* Clear the list of modules which affect this module, by freeing the array
@@ -443,22 +428,14 @@ lu_ent_del_int(GArray *list, const char *attr, const GValue *value)
 	g_return_if_fail(strlen(attr) > 0);
 	dest = lu_ent_get_int(list, attr);
 	if (dest != NULL) {
-		char *svalue;
-
-		svalue = g_strdup_value_contents(value);
 		for (i = 0; i < dest->n_values; i++) {
 			GValue *tvalue;
-			char *tmp;
 
 			tvalue = g_value_array_get_nth(dest, i);
-			tmp = g_strdup_value_contents(tvalue);
-			if (strcmp(tmp, svalue) == 0) {
-				g_free(tmp);
+			if (G_VALUE_TYPE(value) == G_VALUE_TYPE(tvalue)
+			    && lu_values_equal(value, tvalue))
 				break;
-			}
-			g_free(tmp);
 		}
-		g_free(svalue);
 		if (i < dest->n_values) {
 			g_value_array_remove(dest, i);
 			if (dest->n_values == 0)
