@@ -15,7 +15,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: modules.c,v 1.17 2005/10/06 23:02:11 mitr Exp $"
+#ident "$Id: modules.c,v 1.18 2007/09/10 12:47:36 mitr Exp $"
 
 #ifdef HAVE_CONFIG_H
 #include "../config.h"
@@ -46,7 +46,7 @@ load_one_module(struct lu_context *ctx, const char *module_dir,
 	g_free(tmp);
 
 	/* Open the module. */
-	handle = g_module_open(module_file, 0);
+	handle = g_module_open(module_file, G_MODULE_BIND_LOCAL);
 	if (handle == NULL) {
 		/* If the open failed, we return an error. */
 		lu_error_new(error, lu_error_module_load, "%s",
