@@ -15,7 +15,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: prompt.c,v 1.8 2005/10/09 05:15:47 mitr Exp $"
+#ident "$Id: prompt.c,v 1.9 2007/02/15 12:06:12 mitr Exp $"
 
 #ifdef HAVE_CONFIG_H
 #include "../config.h"
@@ -44,7 +44,7 @@ lu_prompt_console(struct lu_prompt *prompts, int count, gpointer calldata,
 	}
 
 	for (i = 0; i < count; i++) {
-		char buf[LINE_MAX];
+		char buf[LINE_MAX], *p;
 		struct termios otermios, ntermios;
 
 		if (prompts[i].prompt) {
@@ -90,14 +90,12 @@ lu_prompt_console(struct lu_prompt *prompts, int count, gpointer calldata,
 			g_print("\n");
 		}
 
-		if (strchr(buf, '\r')) {
-			char *p = strchr(buf, '\r');
+		p = strchr(buf, '\r');
+		if (p != NULL)
 			*p = '\0';
-		}
-		if (strchr(buf, '\n')) {
-			char *p = strchr(buf, '\n');
+		p = strchr(buf, '\n');
+		if (p != NULL)
 			*p = '\0';
-		}
 
 		prompts[i].value = (strlen(buf) > 0) ?
 			g_strdup(buf) :
