@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002, 2004, 2005, 2006 Red Hat, Inc.
+/* Copyright (C) 2000-2002, 2004, 2005, 2006, 2007 Red Hat, Inc.
  *
  * This is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Library General Public License as published by
@@ -15,7 +15,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: user.c,v 1.81 2006/03/06 03:02:52 mitr Exp $"
+#ident "$Id: user.c,v 1.85 2007/06/15 11:56:36 mitr Exp $"
 
 #ifdef HAVE_CONFIG_H
 #include "../config.h"
@@ -24,6 +24,7 @@
 #include <errno.h>
 #include <grp.h>
 #include <inttypes.h>
+#include <limits.h>
 #include <pwd.h>
 #include <stddef.h>
 #include <stdio.h>
@@ -802,13 +803,11 @@ run_list(struct lu_context *context,
 		struct lu_module *module;
 		gpointer scratch;
 		GValue *value;
-		char *name;
 		gboolean tsuccess;
 
 		value = g_value_array_get_nth(list, i);
-		name = g_value_dup_string(value);
-		module = g_tree_lookup(context->modules, name);
-		g_free(name);
+		module = g_tree_lookup(context->modules,
+				       g_value_get_string(value));
 		g_assert(module != NULL);
 		scratch = NULL;
 		tsuccess = run_single(context, module, id,
@@ -1426,18 +1425,11 @@ lu_user_setpass(struct lu_context * context, struct lu_ent * ent,
 	ret = lu_dispatch(context, user_setpass, tmp, LU_VALUE_INVALID_ID,
 			  ent, NULL, error);
 	g_free(tmp);
-	if (ret) {
+	if (ret)
 		ret = lu_refresh_user(context, ent, error);
-	}
 	if (ret) {
-		GValue value;
-		lu_ent_clear(ent, LU_SHADOWLASTCHANGE);
-		memset(&value, 0, sizeof(value));
-		g_value_init(&value, G_TYPE_STRING);
-		g_value_set_string(&value,
-				   lu_util_shadow_current_date(ent->cache));
-		lu_ent_add(ent, LU_SHADOWLASTCHANGE, &value);
-		g_value_unset(&value);
+		lu_util_update_shadow_last_change(ent);
+		ret = lu_user_modify(context, ent, error);
 	}
 	return ret;
 }
@@ -1454,18 +1446,11 @@ lu_user_removepass(struct lu_context * context, struct lu_ent * ent,
 
 	ret = lu_dispatch(context, user_removepass, NULL, LU_VALUE_INVALID_ID,
 			  ent, NULL, error);
-	if (ret) {
+	if (ret)
 		ret = lu_refresh_user(context, ent, error);
-	}
 	if (ret) {
-		GValue value;
-		lu_ent_clear(ent, LU_SHADOWLASTCHANGE);
-		memset(&value, 0, sizeof(value));
-		g_value_init(&value, G_TYPE_STRING);
-		g_value_set_string(&value,
-				   lu_util_shadow_current_date(ent->cache));
-		lu_ent_add(ent, LU_SHADOWLASTCHANGE, &value);
-		g_value_unset(&value);
+		lu_util_update_shadow_last_change(ent);
+		ret = lu_user_modify(context, ent, error);
 	}
 	return ret;
 }
@@ -1545,19 +1530,8 @@ lu_group_setpass(struct lu_context * context, struct lu_ent * ent,
 	ret = lu_dispatch(context, group_setpass, tmp, LU_VALUE_INVALID_ID,
 			  ent, NULL, error);
 	g_free(tmp);
-	if (ret) {
+	if (ret)
 		ret = lu_refresh_group(context, ent, error);
-	}
-	if (ret) {
-		GValue value;
-		lu_ent_clear(ent, LU_SHADOWLASTCHANGE);
-		memset(&value, 0, sizeof(value));
-		g_value_init(&value, G_TYPE_STRING);
-		g_value_set_string(&value,
-				   lu_util_shadow_current_date(ent->cache));
-		lu_ent_add(ent, LU_SHADOWLASTCHANGE, &value);
-		g_value_unset(&value);
-	}
 	return ret;
 }
 
@@ -1573,19 +1547,8 @@ lu_group_removepass(struct lu_context * context, struct lu_ent * ent,
 
 	ret = lu_dispatch(context, group_removepass, NULL, LU_VALUE_INVALID_ID,
 			  ent, NULL, error);
-	if (ret) {
+	if (ret)
 		ret = lu_refresh_group(context, ent, error);
-	}
-	if (ret) {
-		GValue value;
-		lu_ent_clear(ent, LU_SHADOWLASTCHANGE);
-		memset(&value, 0, sizeof(value));
-		g_value_init(&value, G_TYPE_STRING);
-		g_value_set_string(&value,
-				   lu_util_shadow_current_date(ent->cache));
-		lu_ent_add(ent, LU_SHADOWLASTCHANGE, &value);
-		g_value_unset(&value);
-	}
 	return ret;
 }
 
@@ -1750,6 +1713,31 @@ lu_get_first_unused_id(struct lu_context *ctx,
 	return id;
 }
 
+/* Replace all instances of OLD in g_malloc()'ed STRING by NEW. */
+static char *
+replace_all(char *string, const char *old, const char *new)
+{
+	char *pos;
+
+	pos = strstr(string, old);
+	if (pos != NULL) {
+		size_t old_len;
+
+		old_len = strlen(old);
+		do {
+			char *p, *prefix;
+
+			prefix = g_strndup(string, pos - string);
+			p = g_strconcat(prefix, new, pos + old_len, NULL);
+			g_free(prefix);
+			g_free(string);
+			string = p;
+			pos = strstr(string, old);
+		} while (pos != NULL);
+	}
+	return string;
+}
+
 static gboolean
 lu_default_int(struct lu_context *context, const char *name,
 	       enum lu_entity_type type, gboolean is_system, struct lu_ent *ent)
@@ -1820,11 +1808,12 @@ lu_default_int(struct lu_context *context, const char *name,
 	if (is_system) {
 		id = 1;
 	} else {
-		cfgkey = g_strdup_printf("%s/%s", top, idkey);
+		cfgkey = g_strconcat(top, "/", idkey, (const gchar *)NULL);
 		val = lu_cfg_read_single(context, cfgkey, NULL);
 		g_free(cfgkey);
 		if (val == NULL) {
-			cfgkey = g_strdup_printf("%s/%s", top, idkeystring);
+			cfgkey = g_strconcat(top, "/", idkeystring,
+					     (const gchar *)NULL);
 			val = lu_cfg_read_single(context, cfgkey, NULL);
 			g_free(cfgkey);
 		}
@@ -1891,18 +1880,9 @@ lu_default_int(struct lu_context *context, const char *name,
 			{LU_EMAIL, G_STRINGIFY_ARG(LU_EMAIL)},
 		};
 
-		intmax_t imax;
-		char *end, *tmp;
+		char *tmp, replacement[sizeof (intmax_t) * CHAR_BIT + 1];
 		const char *key;
-		struct {
-			const char *format;
-			const char *value;
-		} subst[] = {
-			{"%n", name},
-			{"%d", lu_util_shadow_current_date(context->scache)},
-			/* Must be index 2, see below! */
-			{"%u", NULL} /* value set later */
-		};
+		gboolean ok;
 
 		/* Possibly map the key to an internal name. */
 		key = p->data;
@@ -1920,56 +1900,45 @@ lu_default_int(struct lu_context *context, const char *name,
 		}
 
 		/* Generate the key and read the value for the item. */
-		cfgkey = g_strdup_printf("%s/%s", top, (const char *)p->data);
+		cfgkey = g_strconcat(top, "/", (const gchar *)p->data,
+				     (const gchar *)NULL);
 		val = lu_cfg_read_single(context, cfgkey, NULL);
-		g_free(cfgkey);
 
 		/* Create a copy of the value to mess with. */
 		g_assert(val != NULL);
 		tmp = g_strdup(val);
 
-		subst[2].value = g_strdup_printf("%jd", (intmax_t)id);
-		/* Perform substitutions. */
-		for (i = 0; i < G_N_ELEMENTS(subst); i++) {
-			while (strstr(tmp, subst[i].format) != NULL) {
-				char *pre, *post, *tmp2, *where;
+		tmp = replace_all(tmp, "%n", name);
+		sprintf(replacement, "%ld", lu_util_shadow_current_date());
+		tmp = replace_all(tmp, "%d", replacement);
+		sprintf(replacement, "%jd", (intmax_t)id);
+		tmp = replace_all(tmp, "%u", replacement);
 
-				where = strstr(tmp, subst[i].format);
-				pre = g_strndup(tmp, where - tmp);
-				post = g_strdup(where +
-						strlen(subst[i].format));
-				tmp2 = g_strconcat(pre,
-						   subst[i].value,
-						   post,
-						   NULL);
-				g_free(pre);
-				g_free(post);
-				g_free(tmp);
-				tmp = tmp2;
+		ok = lu_value_init_set_attr_from_string(&value, key, tmp,
+							&error);
+		if (ok == FALSE) {
+			if (error == NULL) {
+				/* Whatever this attribute is, default to a
+				   string. */
+				g_value_init(&value, G_TYPE_STRING);
+				g_value_set_string(&value, tmp);
+				ok = TRUE;
+			} else {
+				g_warning(_("Invalid default value of field "
+					    "%s: %s"), cfgkey,
+					  lu_strerror(error));
+				lu_error_free(&error);
 			}
 		}
-		g_free((char *)subst[2].value);
-
-		/* Check if we can represent this value as a number. */
-		errno = 0;
-		imax = strtoimax(tmp, &end, 10);
-		if (errno == 0 && *end == 0 && end != tmp
-		    && (long)imax == imax) {
-			g_value_init(&value, G_TYPE_LONG);
-			g_value_set_long(&value, imax);
-		} else if (errno == 0 && *end == 0 && end != tmp
-			   && (id_t)imax == imax)
-			lu_value_init_set_id(&value, imax);
-		else {
-			g_value_init(&value, G_TYPE_STRING);
-			g_value_set_string(&value, tmp);
-		}
 		g_free(tmp);
+		g_free(cfgkey);
 
-		/* Add the transformed value. */
-		lu_ent_clear(ent, key);
-		lu_ent_add(ent, key, &value);
-		g_value_unset(&value);
+		if (ok != FALSE) {
+			/* Add the transformed value. */
+			lu_ent_clear(ent, key);
+			lu_ent_add(ent, key, &value);
+			g_value_unset(&value);
+		}
 	}
 	if (keys != NULL) {
 		g_list_free(keys);
