@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2000-2002 Red Hat, Inc.
+ * Copyright (C) 2000-2002, 2007, 2008 Red Hat, Inc.
  *
- * This is free software; you can redistribute it and/or modify it under 
+ * This is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: util.c,v 1.13 2005/12/02 17:32:24 mitr Exp $"
+#ident "$Id: util.c,v 1.20 2008/01/09 10:13:46 mitr Exp $"
 
 #ifdef HAVE_CONFIG_H
 #include "../config.h"
@@ -34,6 +34,9 @@
 #include <string.h>
 #include <time.h>
 #include <unistd.h>
+#ifdef WITH_SELINUX
+#include <selinux/selinux.h>
+#endif
 #define LU_DEFAULT_SALT_TYPE "$1$"
 #define LU_DEFAULT_SALT_LEN  8
 #define LU_MAX_LOCK_ATTEMPTS 6
@@ -41,6 +44,9 @@
 #include "user_private.h"
 #include "internal.h"
 
+#define HASH_ROUNDS_MIN 1000
+#define HASH_ROUNDS_MAX 999999999
+
 struct lu_lock {
 	int fd;
 	struct flock lock;
@@ -113,14 +119,18 @@ fill_urandom(char *output, size_t length)
 	return TRUE;
 }
 
-static struct {
-	const char *initial;
+static const struct {
+	const char initial[5];
+	char separator[2];
 	size_t salt_length;
-	const char *separator;
+	gboolean sha_rounds;
 } salt_type_info[] = {
-	{"$1$", 8, "$"},
-	{"$2a$", 8, "$"},     /* FIXME: number of rounds, base64 of 128 bits */
-	{ "", 2, ""},
+	{"$1$", "$", 8, FALSE },
+	/* FIXME: number of rounds, base64 of 128 bits */
+	{"$2a$", "$", 8, FALSE },
+	{"$5$", "$", 16, TRUE },
+	{"$6$", "$", 16, TRUE },
+	{ "", "", 2 },
 };
 
 const char *
@@ -142,20 +152,119 @@ lu_make_crypted(const char *plain, const char *previous)
 
 	g_assert(i < G_N_ELEMENTS(salt_type_info));
 
-	memset(salt, '\0', sizeof(salt));
-	strncpy(salt, salt_type_info[i].initial, len);
+	if (salt_type_info[i].sha_rounds != FALSE
+	    && strncmp(previous + len, "rounds=", strlen("rounds=")) == 0) {
+		const char *start, *end;
+
+		start = previous + len + strlen("rounds=");
+		end = strchr(start, '$');
+		if (end != NULL
+		    && end <= start + strlen(G_STRINGIFY(HASH_ROUNDS_MAX)))
+			len = (end + 1) - previous;
+	}
+
+	g_assert(len + salt_type_info[i].salt_length
+		 + strlen(salt_type_info[i].separator) < sizeof(salt));
+	memcpy(salt, previous, len);
 
-	g_assert(strlen(salt) +
-		 salt_type_info[i].salt_length +
-		 strlen(salt_type_info[i].separator) <
-		 sizeof(salt));
 	if (fill_urandom(salt + len, salt_type_info[i].salt_length) == FALSE)
 		return NULL;
-	strcat(salt, salt_type_info[i].separator);
+	strcpy(salt + len + salt_type_info[i].salt_length,
+	       salt_type_info[i].separator);
 
 	return crypt(plain, salt);
 }
 
+
+static const char *
+parse_hash_rounds(struct lu_context *context, const char *key,
+		  unsigned long *value)
+{
+	const char *s;
+
+	s = lu_cfg_read_single(context, key, NULL);
+	if (s != NULL) {
+		char *end;
+
+		errno = 0;
+		*value = strtoul(s, &end, 10);
+		if (errno != 0 || *end != 0 || end == s) {
+			g_warning("Invalid %s value '%s'", key, s);
+			s = NULL;
+		}
+	}
+	return s;
+}
+
+static unsigned long
+select_hash_rounds(struct lu_context *context)
+{
+	const char *min_s, *max_s;
+	unsigned long min, max, rounds;
+
+	min_s = parse_hash_rounds(context, "defaults/hash_rounds_min", &min);
+	max_s = parse_hash_rounds(context, "defaults/hash_rounds_max", &max);
+	if (min_s == NULL && max_s == NULL)
+		return 0;
+	if (min_s != NULL && max_s != NULL) {
+		if (min <= max) {
+			if (max > HASH_ROUNDS_MAX)
+				/* To avoid overflow in (max + 1) below */
+				max = HASH_ROUNDS_MAX;
+			rounds = g_random_int_range(min, max + 1);
+		} else
+			rounds = min;
+	} else if (min_s != NULL)
+		rounds = min;
+	else /* max_s != NULL */
+		rounds = max;
+	if (rounds < HASH_ROUNDS_MIN)
+		rounds = HASH_ROUNDS_MIN;
+	else if (rounds > HASH_ROUNDS_MAX)
+		rounds = HASH_ROUNDS_MAX;
+	return rounds;
+}
+
+char *
+lu_util_default_salt_specifier(struct lu_context *context)
+{
+	static const struct {
+		const char *name, *initializer;
+		gboolean sha_rounds;
+	} salt_types[] = {
+		{ "des", "", FALSE },
+		{ "md5", "$1$", FALSE },
+		{ "blowfish", "$2a$", FALSE },
+		{ "sha256", "$5$", TRUE },
+		{ "sha512", "$6$", TRUE },
+	};
+
+	const char *salt_type;
+	size_t i;
+
+	g_return_val_if_fail(context != NULL, g_strdup(""));
+
+	salt_type = lu_cfg_read_single(context, "defaults/crypt_style", "des");
+
+	for (i = 0; i < G_N_ELEMENTS(salt_types); i++) {
+		if (strcasecmp(salt_types[i].name, salt_type) == 0)
+			goto found;
+	}
+	return g_strdup("");
+
+found:
+	if (salt_types[i].sha_rounds != FALSE) {
+		unsigned long rounds;
+
+		rounds = select_hash_rounds(context);
+		if (rounds != 0)
+			return g_strdup_printf("%srounds=%lu$",
+					       salt_types[i].initializer,
+					       rounds);
+	}
+	return g_strdup(salt_types[i].initializer);
+}
+
 gpointer
 lu_util_lock_obtain(int fd, struct lu_error ** error)
 {
@@ -172,9 +281,8 @@ lu_util_lock_obtain(int fd, struct lu_error ** error)
 	do {
 		ret->fd = fd;
 		ret->lock.l_type = F_RDLCK;
-		if (write(ret->fd, NULL, 0) == 0) {
-			ret->lock.l_type |= F_WRLCK;
-		}
+		if (write(ret->fd, NULL, 0) == 0)
+			ret->lock.l_type = F_WRLCK;
 		i = fcntl(ret->fd, F_SETLK, &ret->lock);
 		if ((i == -1) && ((errno == EINTR) || (errno == EAGAIN))) {
 			struct timeval tv;
@@ -557,27 +665,115 @@ err:
 	return ret;
 }
 
-char *
-lu_util_shadow_current_date(struct lu_string_cache *cache)
+long
+lu_util_shadow_current_date(void)
 {
-	struct tm gmt;
+	const struct tm *gmt;
 	time_t now;
-	char buf[LINE_MAX];
 	GDate *today, *epoch;
 	long days;
 
 	time(&now);
-	gmt = *(gmtime(&now));
+	gmt = gmtime(&now);
 
-	today =
-	    g_date_new_dmy(gmt.tm_mday, gmt.tm_mon + 1,
-			   gmt.tm_year + 1900);
+	today = g_date_new_dmy(gmt->tm_mday, gmt->tm_mon + 1,
+			       gmt->tm_year + 1900);
 	epoch = g_date_new_dmy(1, 1, 1970);
 	days = g_date_get_julian(today) - g_date_get_julian(epoch);
 	g_date_free(today);
 	g_date_free(epoch);
 
-	snprintf(buf, sizeof(buf), "%ld", days);
-
-	return cache->cache(cache, buf);
+	return days;
 }
+
+/* Set the shadow last-changed field to today's date. */
+void
+lu_util_update_shadow_last_change(struct lu_ent *ent)
+{
+	GValue value;
+
+	memset(&value, 0, sizeof(value));
+	g_value_init(&value, G_TYPE_LONG);
+	g_value_set_long(&value, lu_util_shadow_current_date());
+	lu_ent_clear(ent, LU_SHADOWLASTCHANGE);
+	lu_ent_add(ent, LU_SHADOWLASTCHANGE, &value);
+	g_value_unset(&value);
+}
+
+#ifdef WITH_SELINUX
+/* Store current fscreate context to ctx. */
+gboolean
+lu_util_fscreate_save(security_context_t *ctx, struct lu_error **error)
+{
+	if (is_selinux_enabled() > 0 && getfscreatecon(ctx) < 0) {
+		lu_error_new(error, lu_error_generic,
+			     _("couldn't get default security context: %s"),
+			     strerror(errno));
+		return FALSE;
+	}
+	return TRUE;
+}
+
+/* Restore fscreate context from ctx, and free it. */
+void
+lu_util_fscreate_restore(security_context_t ctx)
+{
+	if (is_selinux_enabled() > 0) {
+		(void)setfscreatecon(ctx);
+		if (ctx)
+			freecon(ctx);
+	}
+}
+
+/* Set fscreate context from context of file. */
+gboolean
+lu_util_fscreate_from_file(const char *file, struct lu_error **error)
+{
+	if (is_selinux_enabled() > 0) {
+		security_context_t ctx;
+
+		if (getfilecon(file, &ctx) < 0) {
+			lu_error_new(error, lu_error_stat,
+				     _("couldn't get security context of "
+				       "`%s': %s"), file, strerror(errno));
+			return FALSE;
+		}
+		if (setfscreatecon(ctx) < 0) {
+			lu_error_new(error, lu_error_generic,
+				     _("couldn't set default security context "
+				       "to `%s': %s"), ctx, strerror(errno));
+			freecon(ctx);
+			return FALSE;
+		}
+		freecon(ctx);
+	}
+	return TRUE;
+}
+
+/* Set fscreate context for creating a file at path, with file type specified
+   by mode. */
+gboolean
+lu_util_fscreate_for_path(const char *path, mode_t mode,
+			  struct lu_error **error)
+{
+	if (is_selinux_enabled() > 0) {
+		security_context_t ctx;
+
+		if (matchpathcon(path, mode, &ctx) < 0) {
+			lu_error_new(error, lu_error_stat,
+				     _("couldn't determine security context "
+				       "for `%s': %s"), path, strerror(errno));
+			return FALSE;
+		}
+		if (setfscreatecon(ctx) < 0) {
+			lu_error_new(error, lu_error_generic,
+				     _("couldn't set default security context "
+				       "to `%s': %s"), ctx, strerror(errno));
+			freecon(ctx);
+			return FALSE;
+		}
+		freecon(ctx);
+	}
+	return TRUE;
+}
+#endif
