@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002, 2004, 2005 Red Hat, Inc.
+ * Copyright (C) 2000-2002, 2004, 2005, 2008 Red Hat, Inc.
  *
  * This is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Library General Public License as published by
@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: ldap.c,v 1.72 2006/03/11 23:11:12 mitr Exp $"
+#ident "$Id: ldap.c,v 1.78 2008/01/09 10:13:46 mitr Exp $"
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -37,8 +37,6 @@
 #include <sasl/sasl.h>
 #include "../lib/user_private.h"
 
-#include "default.-c"
-
 #undef  DEBUG
 #define LOCKCHAR '!'
 #define LOCKSTRING "!"
@@ -146,6 +144,7 @@ struct lu_ldap_context {
 	struct lu_module *module;		/* The module's structure. */
 	struct lu_prompt prompts[LU_LDAP_MAX];	/* Questions and answers. */
 	gboolean bind_simple, bind_sasl;	/* What kind of bind to use. */
+	char *sasl_mechanism;	/* What sasl mechanism to use. */
 	const char *user_branch, *group_branch;	/* Cached config values */
 	char **mapped_user_attributes, **mapped_group_attributes;
 	LDAP *ldap;				/* The connection. */
@@ -234,37 +233,49 @@ connect_server(struct lu_ldap_context *context, struct lu_error **error)
 
 /* Authentication callback. */
 static int
-interact(LDAP *ld, unsigned flags, void *defs, void *xinteract_data)
+interact(LDAP *ld, unsigned flags, void *defs, void *xres)
 {
-	sasl_interact_t *interact_data;
+	sasl_interact_t *res;
 	struct lu_ldap_context *ctx = (struct lu_ldap_context*) defs;
 	int i, retval = LDAP_SUCCESS;
 
 	(void)ld;
 	(void)flags;
-	interact_data = xinteract_data;
-	for(i = 0; interact_data && (interact_data[i].id != SASL_CB_LIST_END);
-	    i++) {
-		interact_data[i].result = NULL;
-		interact_data[i].len = 0;
-		switch(interact_data[i].id) {
-			case SASL_CB_USER:
-				interact_data[i].result = ctx->prompts[LU_LDAP_AUTHUSER].value ?: "";
-				interact_data[i].len = strlen(interact_data[i].result);
+	res = xres;
+	for(i = 0; res && res[i].id != SASL_CB_LIST_END; i++) {
+		res[i].result = NULL;
+		switch(res[i].id) {
+		case SASL_CB_USER:
+			res[i].result = ctx->prompts[LU_LDAP_AUTHUSER].value;
+			if (res[i].result == NULL)
+				res[i].result = "";
 #ifdef DEBUG
-				g_print("Sending SASL user `%s'.\n", (char*)interact_data[i].result);
+			g_print("Sending SASL user `%s'.\n",
+				(const char *)res[i].result);
 #endif
-				break;
-			case SASL_CB_AUTHNAME:
-				interact_data[i].result = ctx->prompts[LU_LDAP_AUTHZUSER].value;
-				interact_data[i].len = strlen(interact_data[i].result);
+			break;
+		case SASL_CB_AUTHNAME:
+			res[i].result = ctx->prompts[LU_LDAP_AUTHZUSER].value;
 #ifdef DEBUG
-				g_print("Sending SASL auth user `%s'.\n", (char*)interact_data[i].result);
+			g_print("Sending SASL auth user `%s'.\n",
+				(const char *)res[i].result);
 #endif
-				break;
-			default:
-				retval = LDAP_OTHER;
+			break;
+		case SASL_CB_GETREALM:
+			/* Always tell sasl to find it on its own. */
+			res[i].result = "";
+			break;
+		default:
+#ifdef DEBUG
+			g_print("Unhandled SASL Intreractive option `%d'.\n",
+				res[i].id);
+#endif
+			retval = LDAP_OTHER;
 		}
+		if (res[i].result != NULL)
+			res[i].len = strlen(res[i].result);
+		else
+			res[i].len = 0;
 	}
 	return retval;
 }
@@ -335,10 +346,10 @@ bind_server(struct lu_ldap_context *context, struct lu_error **error)
 #ifdef DEBUG
 			g_print("Attempting SASL bind to `%s'.\n", binddn);
 #endif
-			ret = ldap_sasl_interactive_bind_s(ldap, binddn, NULL,
+			ret = ldap_sasl_interactive_bind_s(ldap, binddn,
+							   context->sasl_mechanism,
 							   NULL, NULL,
-							   LDAP_SASL_INTERACTIVE |
-							   LDAP_SASL_QUIET,
+							   LDAP_SASL_AUTOMATIC,
 							   interact,
 							   context);
 			if (ret != LDAP_SUCCESS) {
@@ -353,9 +364,9 @@ bind_server(struct lu_ldap_context *context, struct lu_error **error)
 #endif
 			ret = ldap_sasl_interactive_bind_s(ldap,
 							   generated_binddn,
-							   NULL, NULL, NULL,
-							   LDAP_SASL_INTERACTIVE |
-							   LDAP_SASL_QUIET,
+							   context->sasl_mechanism,
+							   NULL, NULL,
+							   LDAP_SASL_AUTOMATIC,
 							   interact, context);
 			if (ret != LDAP_SUCCESS
 			    && first_failure == LDAP_SUCCESS) {
@@ -447,8 +458,9 @@ lu_ldap_base(struct lu_module *module, const char *branch)
 
 	/* Generate the branch DN. */
 	if (strlen(branch) != 0)
-		tmp = g_strdup_printf("%s,%s", branch,
-				      context->prompts[LU_LDAP_BASEDN].value);
+		tmp = g_strconcat(branch, ",",
+				  context->prompts[LU_LDAP_BASEDN].value,
+				  (const gchar *)NULL);
 	else
 		tmp = g_strdup(context->prompts[LU_LDAP_BASEDN].value);
 
@@ -655,7 +667,8 @@ lu_ldap_lookup(struct lu_module *module,
 				lu_ent_clear_current(ent, attr);
 				for (j = 0; values[j]; j++) {
 					char *val;
-					intmax_t imax;
+					gboolean ok;
+					struct lu_error *error;
 
 					val = g_strndup(values[j]->bv_val,
 							values[j]->bv_len);
@@ -663,30 +676,18 @@ lu_ldap_lookup(struct lu_module *module,
 					g_print("Got `%s' = `%s'.\n", attr,
 						val);
 #endif
-					/* Check if the value is numeric. */
-					errno = 0;
-					imax = strtoimax(val, &p, 10);
-					if (errno == 0 && *p == '\0'
-					    && p != val
-					    && (long)imax == imax) {
-						/* If it's a number, use a
-						 * long. */
-						g_value_init(&value, G_TYPE_LONG);
-						g_value_set_long(&value, imax);
-					} else if (errno == 0 && *p == '\0'
-						   && p != val
-						   && (id_t)imax == imax)
-						lu_value_init_set_id(&value,
-								     imax);
-					else {
-						/* Otherwise it's a string. */
-						g_value_init(&value, G_TYPE_STRING);
-						g_value_set_string(&value, val);
+					error = NULL;
+					ok = lu_value_init_set_attr_from_string
+						(&value, attr, val, &error);
+					if (ok == FALSE) {
+						g_assert(error != NULL);
+						g_warning(lu_strerror(error));
+						lu_error_free(&error);
+					} else {
+						lu_ent_add_current(ent, attr,
+								   &value);
+						g_value_unset(&value);
 					}
-					/* Add this value, and then clear the
-					 * value structure. */
-					lu_ent_add_current(ent, attr, &value);
-					g_value_unset(&value);
 					g_free(val);
 				}
 				ldap_value_free_len(values);
@@ -731,19 +732,15 @@ lu_ldap_user_lookup_id(struct lu_module *module, uid_t uid,
 		       struct lu_ent *ent, struct lu_error **error)
 {
 	struct lu_ldap_context *ctx;
-	gboolean ret;
-	gchar *uid_string;
+	char uid_string[sizeof (uid) * CHAR_BIT + 1];
 
 	LU_ERROR_CHECK(error);
 	ctx = module->module_context;
-	uid_string = g_strdup_printf("%jd", (intmax_t)uid);
-	ret = lu_ldap_lookup(module, "uidNumber",
-			     uid_string, ent, NULL, ctx->user_branch,
-			     "("OBJECTCLASS"="POSIXACCOUNT")",
-			     lu_ldap_user_attributes, lu_user, error);
-	g_free(uid_string);
-
-	return ret;
+	sprintf(uid_string, "%jd", (intmax_t)uid);
+	return lu_ldap_lookup(module, "uidNumber", uid_string, ent, NULL,
+			      ctx->user_branch,
+			      "("OBJECTCLASS"="POSIXACCOUNT")",
+			      lu_ldap_user_attributes, lu_user, error);
 }
 
 /* Look up a group by name. */
@@ -766,18 +763,15 @@ lu_ldap_group_lookup_id(struct lu_module *module, gid_t gid,
 			struct lu_ent *ent, struct lu_error **error)
 {
 	struct lu_ldap_context *ctx;
-	gboolean ret;
-	gchar *gid_string;
+	char gid_string[sizeof (gid) * CHAR_BIT + 1];
 
 	LU_ERROR_CHECK(error);
 	ctx = module->module_context;
-	gid_string = g_strdup_printf("%jd", (intmax_t)gid);
-	ret = lu_ldap_lookup(module, "gidNumber", gid_string, ent, NULL,
-			     ctx->group_branch, "("OBJECTCLASS"="POSIXGROUP")",
-			     lu_ldap_group_attributes, lu_group, error);
-	g_free(gid_string);
-
-	return ret;
+	sprintf(gid_string, "%jd", (intmax_t)gid);
+	return lu_ldap_lookup(module, "gidNumber", gid_string, ent, NULL,
+			      ctx->group_branch,
+			      "("OBJECTCLASS"="POSIXGROUP")",
+			      lu_ldap_group_attributes, lu_group, error);
 }
 
 /* Compare the contents of two GValueArrays, and return TRUE if they contain
@@ -1220,7 +1214,11 @@ static void
 dump_mods(LDAPMod ** mods)
 {
 	size_t i;
-	g_assert(mods != NULL);
+
+	if (mods == NULL) {
+		g_print("NULL modifications");
+		return;
+	}
 	for (i = 0; mods[i]; i++) {
 		g_print("%s (%d)\n", mods[i]->mod_type, mods[i]->mod_op);
 		if (mods[i]->mod_values) {
@@ -1307,7 +1305,7 @@ lu_ldap_set(struct lu_module *module, enum lu_entity_type type, int add,
 	char *name_string;
 	const char *dn, *namingAttr;
 	int err;
-	gboolean ret = FALSE;
+	gboolean ret;
 	struct lu_ldap_context *ctx;
 
 	g_assert(module != NULL);
@@ -1348,40 +1346,31 @@ lu_ldap_set(struct lu_module *module, enum lu_entity_type type, int add,
 	dn = lu_ldap_ent_to_dn(module, namingAttr, name_string, branch);
 	g_free(name_string);
 
-	/* Get the list of changes needed. */
-	if (add)
-		mods = get_ent_adds(dn, ent);
-	else
-		mods = get_ent_mods(ent, namingAttr);
-	if (mods == NULL) {
-		lu_error_new(error, lu_error_generic,
-			     _("could not convert internal data to LDAPMods"));
-		return FALSE;
-	}
 	if (add) {
+		mods = get_ent_adds(dn, ent);
 #ifdef DEBUG
 		dump_mods(mods);
 		g_message("Adding `%s'.\n", dn);
 #endif
 		err = ldap_add_ext_s(ctx->ldap, dn, mods, NULL, NULL);
-		if (err == LDAP_SUCCESS)
-			ret = TRUE;
-		else {
+		if (err != LDAP_SUCCESS) {
 			lu_error_new(error, lu_error_write,
 				     _("error creating a LDAP directory "
 				       "entry: %s"), ldap_err2string(err));
+			ret = FALSE;
 			goto err_mods;
 		}
 	} else {
+		mods = get_ent_mods(ent, namingAttr);
 #ifdef DEBUG
 		dump_mods(mods);
 		g_message("Modifying `%s'.\n", dn);
 #endif
-		/* Attempt the modify operation. */
-		err = ldap_modify_ext_s(ctx->ldap, dn, mods, NULL, NULL);
-		if (err == LDAP_SUCCESS)
-			ret = TRUE;
-		else {
+		/* Attempt the modify operation.  The Fedora Directory server
+		   rejects modify operations with no modifications. */
+		if (mods != NULL && mods[0] != NULL) {
+			err = ldap_modify_ext_s(ctx->ldap, dn, mods, NULL,
+						NULL);
 			if (err == LDAP_OBJECT_CLASS_VIOLATION) {
 				/* AAAARGH!  The application decided it wanted
 				 * to add some new attributes!  Damage
@@ -1390,13 +1379,12 @@ lu_ldap_set(struct lu_module *module, enum lu_entity_type type, int add,
 				err = ldap_modify_ext_s(ctx->ldap, dn, mods,
 							NULL, NULL);
 			}
-			if (err == LDAP_SUCCESS)
-				ret = TRUE;
-			else {
+			if (err != LDAP_SUCCESS) {
 				lu_error_new(error, lu_error_write,
 					     _("error modifying LDAP "
 					       "directory entry: %s"),
 					     ldap_err2string(err));
+				ret = FALSE;
 				goto err_mods;
 			}
 		}
@@ -1405,7 +1393,6 @@ lu_ldap_set(struct lu_module *module, enum lu_entity_type type, int add,
 		if (arrays_equal(name, old_name) == FALSE) {
 			char *tmp1, *tmp2;
 
-			ret = FALSE;
 			/* Format the name to rename it to. */
 			value = g_value_array_get_nth(name, 0);
 			tmp1 = lu_value_strdup(value);
@@ -1417,17 +1404,17 @@ lu_ldap_set(struct lu_module *module, enum lu_entity_type type, int add,
 			err = ldap_rename_s(ctx->ldap, dn, tmp2, NULL, TRUE,
 					    NULL, NULL);
 			g_free(tmp2);
-			if (err == LDAP_SUCCESS)
-				ret = TRUE;
-			else {
+			if (err != LDAP_SUCCESS) {
 				lu_error_new(error, lu_error_write,
 					     _("error renaming LDAP directory "
 					       "entry: %s"),
 					     ldap_err2string(err));
+				ret = FALSE;
 				goto err_mods;
 			}
 		}
 	}
+	ret = TRUE;
 
  err_mods:
 	free_ent_mods(mods);
@@ -1545,8 +1532,11 @@ lu_ldap_handle_lock(struct lu_module *module, struct lu_ent *ent,
 	/* We only know how to lock crypted passwords, so crypt it if it
 	 * isn't already. */
 	if (!g_str_has_prefix(oldpassword, LU_CRYPTED)) {
-		tmp = lu_make_crypted(oldpassword,
-				      lu_common_default_salt_specifier(module));
+		char *salt;
+
+		salt = lu_util_default_salt_specifier(module->lu_context);
+		tmp = lu_make_crypted(oldpassword, salt);
+		g_free(salt);
 		if (tmp == NULL) {
 			lu_error_new(error, lu_error_generic,
 				     _("error encrypting password"));
@@ -1565,27 +1555,23 @@ lu_ldap_handle_lock(struct lu_module *module, struct lu_ent *ent,
 			result = g_strdup_printf("%s%c%s", LU_CRYPTED,
 						 LOCKCHAR, result);
 		else
-			result = g_strdup_printf("%s%s", LU_CRYPTED, result);
-		break;
-	case LO_UNLOCK:
-		if (result[0] == LOCKCHAR)
-			result = g_strdup_printf("%s%s", LU_CRYPTED,
-						 result + 1);
-		else
-			result = g_strdup_printf("%s%s", LU_CRYPTED, result);
+			result = g_strconcat(LU_CRYPTED, result,
+					     (const gchar *)NULL);
 		break;
 	case LO_UNLOCK_NONEMPTY:
-		if (result[0] == LOCKCHAR) {
-			if (result[1] == '\0') {
-				lu_error_new(error, lu_error_unlock_empty,
-					     NULL);
-				g_free(oldpassword);
-				return FALSE;
-			}
-			result = g_strdup_printf("%s%s", LU_CRYPTED,
-						 result + 1);
-		} else
-			result = g_strdup_printf("%s%s", LU_CRYPTED, result);
+		if (result[0] == LOCKCHAR && result[1] == '\0') {
+			lu_error_new(error, lu_error_unlock_empty, NULL);
+			g_free(oldpassword);
+			return FALSE;
+		}
+		/* Fall through */
+	case LO_UNLOCK:
+		if (result[0] == LOCKCHAR)
+			result = g_strconcat(LU_CRYPTED, result + 1,
+					     (const gchar *)NULL);
+		else
+			result = g_strconcat(LU_CRYPTED, result,
+					     (const gchar *)NULL);
 		break;
 	default:
 		g_assert_not_reached();
@@ -1814,12 +1800,16 @@ lu_ldap_setpass(struct lu_module *module, const char *namingAttr,
 		addvalues[0] = (char *)password;
 	else {
 		const char *crypted;
-		char *tmp;
+		char *salt, *tmp;
 
-		crypted =
-		    lu_make_crypted(password, previous
-				    ? (previous + strlen(LU_CRYPTED)) :
-				    lu_common_default_salt_specifier(module));
+		if (previous != NULL)
+			salt = previous + strlen(LU_CRYPTED);
+		else
+			salt = lu_util_default_salt_specifier(module
+							      ->lu_context);
+		crypted = lu_make_crypted(password, salt);
+		if (previous == NULL)
+			g_free(salt);
 		if (crypted == NULL) {
 			lu_error_new(error, lu_error_generic,
 				     _("error encrypting password"));
@@ -2278,12 +2268,12 @@ lu_ldap_users_enumerate_by_group(struct lu_module *module,
 	struct lu_ldap_context *ctx;
 	GValueArray *primaries;
 	GValue *value;
-	char *grp;
+	char grp[sizeof (gid) * CHAR_BIT + 1];
 	size_t i;
 
 	LU_ERROR_CHECK(error);
 	ctx = module->module_context;
-	grp = g_strdup_printf("%jd", (intmax_t)gid);
+	sprintf(grp, "%jd", (intmax_t)gid);
 
 	primaries = lu_ldap_enumerate(module, "gidNumber", grp, "uid",
 				      ctx->user_branch, error);
@@ -2307,7 +2297,6 @@ lu_ldap_users_enumerate_by_group(struct lu_module *module,
 			g_value_get_string(value));
 	}
 #endif
-	g_free(grp);
 	return primaries;
 }
 
@@ -2427,6 +2416,7 @@ lu_ldap_close_module(struct lu_module *module)
 			ctx->prompts[i].free_value(ctx->prompts[i].value);
 		}
 	}
+	g_free(ctx->sasl_mechanism);
 	g_free(ctx->mapped_user_attributes);
 	g_free(ctx->mapped_group_attributes);
 	g_free(ctx);
@@ -2450,7 +2440,6 @@ libuser_ldap_init(struct lu_context *context, struct lu_error **error)
 	struct lu_module *ret;
 	struct lu_ldap_context *ctx;
 	struct lu_prompt prompts[G_N_ELEMENTS(ctx->prompts)];
-	char *user;
 	const char *bind_type;
 	char **bind_types;
 	size_t i;
@@ -2487,13 +2476,11 @@ libuser_ldap_init(struct lu_context *context, struct lu_error **error)
 	ctx->prompts[LU_LDAP_PASSWORD].prompt = N_("LDAP Bind Password");
 	ctx->prompts[LU_LDAP_PASSWORD].visible = FALSE;
 
-	user = getuser();
-
 	ctx->prompts[LU_LDAP_AUTHUSER].key = "ldap/user";
 	ctx->prompts[LU_LDAP_AUTHUSER].prompt = N_("LDAP SASL User");
 	ctx->prompts[LU_LDAP_AUTHUSER].visible = TRUE;
 	ctx->prompts[LU_LDAP_AUTHUSER].default_value =
-		lu_cfg_read_single(context, "ldap/user", user);
+		lu_cfg_read_single(context, "ldap/user", "");
 
 	ctx->prompts[LU_LDAP_AUTHZUSER].key = "ldap/authuser";
 	ctx->prompts[LU_LDAP_AUTHZUSER].prompt =
@@ -2502,11 +2489,6 @@ libuser_ldap_init(struct lu_context *context, struct lu_error **error)
 	ctx->prompts[LU_LDAP_AUTHZUSER].default_value =
 		lu_cfg_read_single(context, "ldap/authuser", "");
 
-	if (user) {
-		free(user);
-		user = NULL;
-	}
-
 	/* Try to be somewhat smart and allow the user to specify which bind
 	 * type to use, which should prevent us from asking for information
 	 * we can be certain we don't have a use for. */
@@ -2518,6 +2500,11 @@ libuser_ldap_init(struct lu_context *context, struct lu_error **error)
 		} else
 		if (g_ascii_strcasecmp(bind_types[i], "sasl") == 0) {
 			ctx->bind_sasl = TRUE;
+			ctx->sasl_mechanism = NULL;
+		}
+		if (g_ascii_strncasecmp(bind_types[i], "sasl/", 5) == 0) {
+			ctx->bind_sasl = TRUE;
+			ctx->sasl_mechanism = g_strdup(bind_types[i] + 5);
 		}
 	}
 	g_strfreev(bind_types);
