@@ -15,7 +15,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: admin.c,v 1.43 2006/05/01 04:51:21 mitr Exp $"
+#ident "$Id: admin.c,v 1.47 2007/10/11 03:18:34 mitr Exp $"
 
 #include <Python.h>
 #ifdef HAVE_CONFIG_H
@@ -38,10 +38,6 @@ static PyMethodDef libuser_admin_methods[];
 static PyTypeObject AdminType;
 #define Admin_Check(__x) ((__x)->ob_type == &AdminType)
 
-static struct libuser_admin *libuser_admin_new(PyObject *self,
-					       PyObject *args,
-					       PyObject *kwargs);
-
 /* Destroy the object. */
 static void
 libuser_admin_destroy(PyObject *self)
@@ -64,7 +60,7 @@ libuser_admin_destroy(PyObject *self)
 		me->prompt_data[i] = NULL;
 	}
 	/* Delete the python object. */
-	PyMem_DEL(self);
+	PyObject_DEL(self);
 	DEBUG_EXIT;
 }
 
@@ -96,7 +92,7 @@ libuser_admin_getattr(PyObject *self, char *name)
 
 /* Set an attribute in the admin object. */
 static int
-libuser_admin_setattr(PyObject *self, const char *attr, PyObject *args)
+libuser_admin_setattr(PyObject *self, char *attr, PyObject *args)
 {
 	struct libuser_admin *me = (struct libuser_admin *) self;
 
@@ -113,7 +109,8 @@ libuser_admin_setattr(PyObject *self, const char *attr, PyObject *args)
 			Py_DECREF(me->prompt_data[1]);
 			me->prompt_data[0] = args;
 			Py_INCREF(me->prompt_data[0]);
-			me->prompt_data[1] = Py_BuildValue("");
+			me->prompt_data[1] = Py_None;
+			Py_INCREF(me->prompt_data[0]);
 		}
 		/* If it's a tuple, the first item is the function, and the
 		 * rest are arguments to pass to it. */
@@ -174,7 +171,7 @@ libuser_admin_lookup_user_name(PyObject *self, PyObject *args,
 			lu_error_free(&error);
 		lu_ent_free(ent);
 		DEBUG_EXIT;
-		return Py_BuildValue("");
+		Py_RETURN_NONE;
 	}
 }
 
@@ -211,7 +208,7 @@ libuser_admin_lookup_user_id(PyObject *self, PyObject *args,
 			lu_error_free(&error);
 		lu_ent_free(ent);
 		DEBUG_EXIT;
-		return Py_BuildValue("");
+		Py_RETURN_NONE;
 	}
 }
 
@@ -244,7 +241,7 @@ libuser_admin_lookup_group_name(PyObject *self, PyObject *args,
 			lu_error_free(&error);
 		lu_ent_free(ent);
 		DEBUG_EXIT;
-		return Py_BuildValue("");
+		Py_RETURN_NONE;
 	}
 }
 
@@ -282,7 +279,7 @@ libuser_admin_lookup_group_id(PyObject *self, PyObject *args,
 			lu_error_free(&error);
 		lu_ent_free(ent);
 		DEBUG_EXIT;
-		return Py_BuildValue("");
+		Py_RETURN_NONE;
 	}
 }
 
@@ -352,7 +349,7 @@ libuser_admin_do_wrap(PyObject *self, struct libuser_entity *ent,
 	if (fn(me->ctx, ent->ent, &error)) {
 		/* It succeeded!  Return truth. */
 		DEBUG_EXIT;
-		return Py_BuildValue("i", 1);
+		return PyInt_FromLong(1);
 	} else {
 		/* It failed.  Build an exception and return an error. */
 		PyErr_SetString(PyExc_RuntimeError, lu_strerror(error));
@@ -413,7 +410,7 @@ libuser_admin_wrap_boolean(PyObject *self, PyObject *args, PyObject *kwargs,
 	if (error != NULL)
 		lu_error_free(&error);
 	DEBUG_EXIT;
-	return Py_BuildValue("i", ret ? 1 : 0);
+	return PyInt_FromLong(ret ? 1 : 0);
 }
 
 /* Wrap the setpass function for either type of entity. */
@@ -444,7 +441,7 @@ libuser_admin_setpass(PyObject *self, PyObject *args, PyObject *kwargs,
 	       &error)) {
 		/* The change succeeded.  Return a truth. */
 		DEBUG_EXIT;
-		return Py_BuildValue("i", 1);
+		return PyInt_FromLong(1);
 	} else {
 		/* The change failed.  Return an error. */
 		PyErr_SetString(PyExc_SystemError, lu_strerror(error));
@@ -521,7 +518,7 @@ libuser_admin_create_home(PyObject *self, PyObject *args,
 				0700, &error)) {
 		/* Success -- return an empty tuple. */
 		DEBUG_EXIT;
-		return Py_BuildValue("i", 1);
+		return PyInt_FromLong(1);
 	} else {
 		/* Failure.  Mark the error. */
 		PyErr_SetString(PyExc_RuntimeError,
@@ -573,7 +570,7 @@ libuser_admin_remove_home(PyObject *self, PyObject *args,
 	if (lu_homedir_remove(dir, &error)) {
 		/* Successfully removed. */
 		DEBUG_EXIT;
-		return Py_BuildValue("i", 1);
+		return PyInt_FromLong(1);
 	} else {
 		/* Removal failed.  You'll have to come back for repeated
 		 * treatments. */
@@ -652,7 +649,7 @@ libuser_admin_move_home(PyObject *self, PyObject *args,
 	if (lu_homedir_move(olddir, newdir, &error)) {
 		/* Success! */
 		DEBUG_EXIT;
-		return Py_BuildValue("i", 1);
+		return PyInt_FromLong(1);
 	} else {
 		/* Failure.  Set an error. */
 		PyErr_SetString(PyExc_RuntimeError,
@@ -688,7 +685,7 @@ libuser_admin_create_remove_mail(PyObject *self, PyObject *args,
 
 	/* Now just pass it to the internal function. */
 	if (lu_mailspool_create_remove(me->ctx, ent->ent, action)) {
-		return Py_BuildValue("i", 1);
+		return PyInt_FromLong(1);
 	} else {
 		PyErr_SetString(PyExc_RuntimeError,
 				_("error creating mail spool for user"));
@@ -740,45 +737,37 @@ libuser_admin_add_user(PyObject *self, PyObject *args, PyObject *kwargs)
 
 	/* Pass the entity object to lu_user_add(). */
 	ret = libuser_admin_do_wrap(self, ent, lu_user_add);
-	if (ret != NULL) {
-		/* If we got a non-NULL response, then it was okay. */
-		if ((mkhomedir != NULL) && (PyObject_IsTrue(mkhomedir))) {
-			PyObject *subargs, *subkwargs;
+	if (ret != NULL && mkhomedir != NULL && PyObject_IsTrue(mkhomedir)) {
+		PyObject *subargs, *subkwargs;
 
-			/* Free the result we got. */
-			if (ret != NULL) {
-				Py_DECREF(ret);
-			}
-			/* Create the user's home directory we need to pass
-			 * the entity structure in a tuple, so create a tuple
-			 * and add just that object to it. */
-			subargs = PyTuple_New(1);
-			Py_INCREF(ent);
-			PyTuple_SetItem(subargs, 0, (PyObject*) ent);
-			/* Create a dictionary for keyword args. */
-			subkwargs = PyDict_New();
-			if (skeleton != NULL) {
-				Py_INCREF(skeleton);
-				PyDict_SetItemString(subkwargs, "skeleton",
-						     skeleton);
-			}
-			/* We'll return the result of the creation call. */
-			ret = libuser_admin_create_home(self, subargs,
-							subkwargs);
-			Py_DECREF(subargs);
-			Py_DECREF(subkwargs);
+		Py_DECREF(ret);
+		/* Create the user's home directory we need to pass the entity
+		   structure in a tuple, so create a tuple * and add just that
+		   object to it. */
+		subargs = PyTuple_New(1);
+		Py_INCREF(ent);
+		PyTuple_SetItem(subargs, 0, (PyObject*) ent);
+		/* Create a dictionary for keyword args. */
+		subkwargs = PyDict_New();
+		if (skeleton != NULL) {
+			Py_INCREF(skeleton);
+			PyDict_SetItemString(subkwargs, "skeleton", skeleton);
 		}
-		/* If we got a non-NULL response, then it was okay. */
-		if ((mkmailspool != NULL) && (PyObject_IsTrue(mkmailspool))) {
-			if (ret != NULL) {
-				Py_DECREF(ret);
-			}
-			if (lu_mailspool_create_remove(context, ent->ent, TRUE) ) {
-				ret = Py_BuildValue("i", 1);
-			} else {
-				/* An exception has been thrown. */
-				ret = NULL;
-			}
+		/* We'll return the result of the creation call. */
+		ret = libuser_admin_create_home(self, subargs, subkwargs);
+		Py_DECREF(subargs);
+		Py_DECREF(subkwargs);
+	}
+	if (ret != NULL && mkmailspool != NULL
+	    && PyObject_IsTrue(mkmailspool)) {
+		Py_DECREF(ret);
+		if (lu_mailspool_create_remove(context, ent->ent, TRUE))
+			ret = PyInt_FromLong(1);
+		else {
+			PyErr_SetString(PyExc_RuntimeError,
+					_("error creating mail spool for "
+					  "user"));
+			ret = NULL;
 		}
 	}
 
@@ -875,29 +864,31 @@ libuser_admin_delete_user(PyObject *self, PyObject *args,
 
 	ret = libuser_admin_do_wrap(self, (struct libuser_entity *)ent,
 				    lu_user_delete);
-	if (ret != NULL) {
-		if ((rmhomedir != NULL) && (PyObject_IsTrue(rmhomedir))) {
-			PyObject *subargs, *subkwargs;
+	if (ret != NULL && rmhomedir != NULL && PyObject_IsTrue(rmhomedir)) {
+		PyObject *subargs, *subkwargs;
 
-			Py_DECREF(ret);
-			subargs = PyTuple_New(1);
-			Py_INCREF(ent);
-			PyTuple_SetItem(subargs, 0, ent);
-			subkwargs = PyDict_New();
-			ret = libuser_admin_remove_home(self, subargs,
-						        subkwargs);
-			Py_DECREF(subargs);
-			Py_DECREF(subkwargs);
-		}
+		Py_DECREF(ret);
+		subargs = PyTuple_New(1);
+		Py_INCREF(ent);
+		PyTuple_SetItem(subargs, 0, ent);
+		subkwargs = PyDict_New();
+		ret = libuser_admin_remove_home(self, subargs, subkwargs);
+		Py_DECREF(subargs);
+		Py_DECREF(subkwargs);
 	}
-	if (ret != NULL) {
-		if ((rmmailspool!= NULL) && (PyObject_IsTrue(rmmailspool))) {
-			struct libuser_entity *entity;
-			Py_DECREF(ret);
-			entity = (struct libuser_entity *)ent;
-			ret = lu_mailspool_create_remove(context, entity->ent, TRUE) ?
-			      Py_BuildValue("i", 1) :
-			      NULL;
+	if (ret != NULL && rmmailspool != NULL
+	    && PyObject_IsTrue(rmmailspool)) {
+		struct libuser_entity *entity;
+
+		Py_DECREF(ret);
+		entity = (struct libuser_entity *)ent;
+		if (lu_mailspool_create_remove(context, entity->ent, TRUE))
+			ret = PyInt_FromLong(1);
+		else {
+			PyErr_SetString(PyExc_RuntimeError,
+					_("error removing mail spool for "
+					  "user"));
+			ret = NULL;
 		}
 	}
 
@@ -1301,16 +1292,15 @@ libuser_admin_enumerate_groups_by_user_full(PyObject *self, PyObject *args,
 #endif
 
 static PyObject *
-libuser_admin_get_first_unused_id_type(struct libuser_admin *self,
+libuser_admin_get_first_unused_id_type(struct libuser_admin *me,
 				       PyObject * args, PyObject * kwargs,
 				       enum lu_entity_type enttype)
 {
 	char *keywords[] = { "start", NULL };
-	struct libuser_admin *me = (struct libuser_admin *) self;
 
 	PY_LONG_LONG start = 500;
 
-	g_return_val_if_fail(self != NULL, NULL);
+	g_return_val_if_fail(me != NULL, NULL);
 
 	DEBUG_ENTRY;
 	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "|L", keywords,
@@ -1324,29 +1314,24 @@ libuser_admin_get_first_unused_id_type(struct libuser_admin *self,
 		return NULL;
 	}
 
-	return Py_BuildValue("L", (PY_LONG_LONG)lu_get_first_unused_id(me->ctx,
-								       enttype,
-								       start));
+	return PyLong_FromLongLong(lu_get_first_unused_id(me->ctx, enttype,
+							  start));
 }
 
 static PyObject *
-libuser_admin_get_first_unused_uid(struct libuser_admin *self, PyObject * args,
-			           PyObject * kwargs)
+libuser_admin_get_first_unused_uid(PyObject *self, PyObject * args,
+				   PyObject *kwargs)
 {
-	return libuser_admin_get_first_unused_id_type(self,
-						      args,
-						      kwargs,
-						      lu_user);
+	return libuser_admin_get_first_unused_id_type
+		((struct libuser_admin *)self, args, kwargs, lu_user);
 }
 
 static PyObject *
-libuser_admin_get_first_unused_gid(struct libuser_admin *self, PyObject * args,
-			           PyObject * kwargs)
+libuser_admin_get_first_unused_gid(PyObject *self, PyObject * args,
+				   PyObject *kwargs)
 {
-	return libuser_admin_get_first_unused_id_type(self,
-						      args,
-						      kwargs,
-						      lu_group);
+	return libuser_admin_get_first_unused_id_type
+		((struct libuser_admin *)self, args, kwargs, lu_group);
 }
 
 static struct PyMethodDef libuser_admin_methods[] = {
@@ -1482,7 +1467,7 @@ static struct PyMethodDef libuser_admin_methods[] = {
 	 METH_VARARGS | METH_KEYWORDS,
 	 "remove a mail spool for a user"},
 
-	{"getUserShells", (PyCFunction) libuser_get_user_shells, 0,
+	{"getUserShells", libuser_get_user_shells, METH_NOARGS,
 	 "return a list of valid shells"},
 
 
@@ -1502,26 +1487,19 @@ static struct PyMethodDef libuser_admin_methods[] = {
 static PyTypeObject AdminType = {
 	PyObject_HEAD_INIT(&PyType_Type)
        	0,
-	"Admin",
-	sizeof(struct libuser_admin),
-	0,
+	"Admin",		/* tp_name */
+	sizeof(struct libuser_admin), /* tp_basicsize */
+	0,			/* tp_itemsize */
 
-	(destructor) libuser_admin_destroy,
-	(printfunc) NULL,
-	(getattrfunc) libuser_admin_getattr,
-	(setattrfunc) libuser_admin_setattr,
-	(cmpfunc) NULL,
-	(reprfunc) NULL,
-
-	(PyNumberMethods *) NULL,
-	(PySequenceMethods *) NULL,
-	(PyMappingMethods *) NULL,
-	(hashfunc) NULL,
-	(ternaryfunc) NULL,
-	(reprfunc) NULL,
+	libuser_admin_destroy,	/* tp_dealloc */
+	NULL,			/* tp_print */
+	libuser_admin_getattr,	/* tp_getattr */
+	libuser_admin_setattr,	/* tp_setattr */
+	NULL,			/* tp_compare */
+	NULL,			/* tp_repr */
 };
 
-static struct libuser_admin *
+static PyObject *
 libuser_admin_new(PyObject *self, PyObject *args, PyObject *kwargs)
 {
 	char *name = getlogin(), *modules = NULL, *create = NULL, *p, *q;
@@ -1575,12 +1553,11 @@ libuser_admin_new(PyObject *self, PyObject *args, PyObject *kwargs)
 				  "promptConsole");
 	}
 
-	if (prompt_data != NULL) {
+	if (prompt_data != NULL)
 		ret->prompt_data[1] = prompt_data;
-		Py_INCREF(ret->prompt_data[1]);
-	} else {
-		ret->prompt_data[1] = Py_BuildValue("");
-	}
+	else
+		ret->prompt_data[1] = Py_None;
+	Py_INCREF(ret->prompt_data[1]);
 
 #ifdef DEBUG_BINDING
 	fprintf(stderr,
@@ -1605,5 +1582,5 @@ libuser_admin_new(PyObject *self, PyObject *args, PyObject *kwargs)
 	ret->ctx = context;
 
 	DEBUG_EXIT;
-	return ret;
+	return self;
 }
