@@ -15,14 +15,15 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: misc.c,v 1.22 2005/10/09 05:15:47 mitr Exp $"
+#ident "$Id: misc.c,v 1.27 2007/01/06 20:40:17 katzj Exp $"
 
 #include <Python.h>
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
-#include <pwd.h>
 #include <grp.h>
+#include <limits.h>
+#include <pwd.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <glib.h>
@@ -33,8 +34,6 @@
 static PyTypeObject PromptType;
 #define Prompt_Check(__x) ((__x)->ob_type == &PromptType)
 
-static struct libuser_prompt *libuser_prompt_new(void);
-
 static gboolean
 libuser_admin_python_prompter(struct lu_prompt *prompts, int count,
 			      gpointer callback_data,
@@ -58,7 +57,8 @@ libuser_admin_python_prompter(struct lu_prompt *prompts, int count,
 		for (i = 0; i < count; i++) {
 			struct libuser_prompt *prompt;
 
-			prompt = libuser_prompt_new();
+			prompt = (struct libuser_prompt *)
+			  libuser_prompt_new(NULL, NULL);
 			prompt->prompt.key = g_strdup(prompts[i].key);
 			prompt->prompt.prompt = g_strdup(prompts[i].prompt);
 			prompt->prompt.domain = g_strdup(prompts[i].domain);
@@ -74,12 +74,14 @@ libuser_admin_python_prompter(struct lu_prompt *prompts, int count,
 			            PyTuple_Size(prompt_data[1]) + 1 : 1);
 		PyTuple_SetItem(tuple, 0, list);
 		if (PyTuple_Check(prompt_data[1])) {
-			for (i = 0; i < PyTuple_Size(prompt_data[1]); i++) {
+			Py_ssize_t j;
+
+			for (j = 0; j < PyTuple_Size(prompt_data[1]); j++) {
 				PyObject *obj;
 
-				obj = PyTuple_GetItem(prompt_data[1], i);
+				obj = PyTuple_GetItem(prompt_data[1], j);
 				Py_INCREF(obj);
-				PyTuple_SetItem(tuple, i + 1, obj);
+				PyTuple_SetItem(tuple, j + 1, obj);
 			}
 		}
 		ret = PyObject_CallObject(prompt_data[0], tuple);
@@ -94,9 +96,10 @@ libuser_admin_python_prompter(struct lu_prompt *prompts, int count,
 		}
 		for (i = 0; i < count; i++) {
 			struct libuser_prompt *prompt;
-			prompt =
-			    (struct libuser_prompt *) PyList_GetItem(list,
-								     i);
+			/* i doesn't have to be Py_ssize_t because count is int
+			   as well. */
+			prompt = (struct libuser_prompt *)PyList_GetItem(list,
+									 i);
 			prompts[i].value = g_strdup(prompt->prompt.value);
 			prompts[i].free_value = g_free;
 		}
@@ -112,7 +115,8 @@ static PyObject *
 libuser_admin_prompt(struct libuser_admin *self, PyObject * args,
 		     PyObject * kwargs, lu_prompt_fn * prompter)
 {
-	int count, i;
+	Py_ssize_t count;
+	int i;
 	PyObject *list = NULL, *moreargs = NULL;
 	struct lu_prompt *prompts;
 	struct lu_error *error = NULL;
@@ -127,9 +131,13 @@ libuser_admin_prompt(struct libuser_admin *self, PyObject * args,
 		DEBUG_EXIT;
 		return NULL;
 	}
-	DEBUG_CALL;
 	count = PyList_Size(list);
-	DEBUG_CALL;
+	if (count > INT_MAX) {
+		PyErr_SetString(PyExc_ValueError, "too many prompts");
+		DEBUG_EXIT;
+		return NULL;
+	}
+	/* i now may be int because count fits in int */
 	for (i = 0; i < count; i++) {
 		PyObject *item;
 
@@ -144,8 +152,6 @@ libuser_admin_prompt(struct libuser_admin *self, PyObject * args,
 		DEBUG_CALL;
 	}
 	DEBUG_CALL;
-	count = PyList_Size(list);
-	DEBUG_CALL;
 	prompts = g_malloc0(count * sizeof(struct lu_prompt));
 	DEBUG_CALL;
 
@@ -173,9 +179,9 @@ libuser_admin_prompt(struct libuser_admin *self, PyObject * args,
 	if (prompter(prompts, count, self->prompt_data, &error) != FALSE) {
 		for (i = 0; i < count; i++) {
 			struct libuser_prompt *obj;
-			obj = (struct libuser_prompt *) PyList_GetItem(list, i);
+			obj = (struct libuser_prompt *)PyList_GetItem(list, i);
 			obj->prompt.value = g_strdup(prompts[i].value ? : "");
-			obj->prompt.free_value = (typeof(obj->prompt.free_value)) g_free;
+			obj->prompt.free_value = g_free;
 			if (prompts[i].value && prompts[i].free_value) {
 				prompts[i].free_value(prompts[i].value);
 				prompts[i].value = NULL;
@@ -184,7 +190,7 @@ libuser_admin_prompt(struct libuser_admin *self, PyObject * args,
 			Py_DECREF(obj);
 		}
 		DEBUG_EXIT;
-		return Py_BuildValue("");
+		Py_RETURN_NONE;
 	} else {
 		if (error != NULL)
 			lu_error_free(&error);
@@ -220,64 +226,72 @@ libuser_admin_prompt_console_quiet(PyObject * self, PyObject * args,
 }
 
 static void
-libuser_prompt_destroy(struct libuser_prompt *self)
+libuser_prompt_destroy(PyObject *self)
 {
+	struct libuser_prompt *me;
+
 	DEBUG_ENTRY;
-	if (self->prompt.value && self->prompt.free_value)
-		self->prompt.free_value(self->prompt.value);
-	g_free((void *)self->prompt.key);
-	g_free((void *)self->prompt.prompt);
-	g_free((void *)self->prompt.domain);
-	g_free((void *)self->prompt.default_value);
-	memset(&self->prompt, 0, sizeof(self->prompt));
-	PyMem_DEL(self);
+	me = (struct libuser_prompt *)self;
+	if (me->prompt.value && me->prompt.free_value)
+		me->prompt.free_value(me->prompt.value);
+	g_free((void *)me->prompt.key);
+	g_free((void *)me->prompt.prompt);
+	g_free((void *)me->prompt.domain);
+	g_free((void *)me->prompt.default_value);
+	memset(&me->prompt, 0, sizeof(me->prompt));
+	PyObject_DEL(me);
 	DEBUG_EXIT;
 }
 
 static PyObject *
-libuser_prompt_getattr(struct libuser_prompt *self, char *attr)
+libuser_prompt_getattr(PyObject *self, char *attr)
 {
+	struct libuser_prompt *me;
+
 	DEBUG_ENTRY;
+	me = (struct libuser_prompt *)self;
 	if (strcmp(attr, "key") == 0) {
 		DEBUG_EXIT;
-		return PyString_FromString(self->prompt.key);
+		return PyString_FromString(me->prompt.key);
 	}
 	if (strcmp(attr, "prompt") == 0) {
 		DEBUG_EXIT;
-		return PyString_FromString(self->prompt.prompt);
+		return PyString_FromString(me->prompt.prompt);
 	}
 	if (strcmp(attr, "domain") == 0) {
 		DEBUG_EXIT;
-		return PyString_FromString(self->prompt.domain ?: "");
+		return PyString_FromString(me->prompt.domain ?: "");
 	}
 	if (strcmp(attr, "visible") == 0) {
 		DEBUG_EXIT;
-		return PyInt_FromLong(self->prompt.visible);
+		return PyInt_FromLong(me->prompt.visible);
 	}
 	if ((strcmp(attr, "default_value") == 0) ||
 	    (strcmp(attr, "defaultValue") == 0)) {
 		DEBUG_EXIT;
-		return self->prompt.
-		    default_value ? PyString_FromString(self->prompt.
-							default_value) :
-		    Py_BuildValue("");
+		if (me->prompt.default_value != NULL)
+			return PyString_FromString(me->prompt.default_value);
+		else
+			Py_RETURN_NONE;
 	}
 	if (strcmp(attr, "value") == 0) {
 		DEBUG_EXIT;
-		return self->prompt.value ? PyString_FromString(self->
-								prompt.
-								value) :
-		    Py_BuildValue("");
+		if (me->prompt.value != NULL)
+			return PyString_FromString(me->prompt.value);
+		else
+			Py_RETURN_NONE;
 	}
 	DEBUG_EXIT;
-	return Py_FindMethod(NULL, (PyObject *) self, attr);
+	return Py_FindMethod(NULL, self, attr);
 }
 
 static int
-libuser_prompt_setattr(struct libuser_prompt *self, const char *attr,
-		       PyObject * args)
+libuser_prompt_setattr(PyObject *self, char *attr, PyObject * args)
 {
+	struct libuser_prompt *me;
+
 	DEBUG_ENTRY;
+	me = (struct libuser_prompt *)self;
 	if (strcmp(attr, "prompt") == 0) {
 		if (!PyString_Check(args)) {
 			PyErr_SetString(PyExc_TypeError,
@@ -285,8 +299,8 @@ libuser_prompt_setattr(struct libuser_prompt *self, const char *attr,
 			DEBUG_EXIT;
 			return -1;
 		}
-		g_free((char *) self->prompt.prompt);
-		self->prompt.prompt = g_strdup(PyString_AsString(args));
+		g_free((char *)me->prompt.prompt);
+		me->prompt.prompt = g_strdup(PyString_AsString(args));
 		DEBUG_EXIT;
 		return 0;
 	}
@@ -297,8 +311,8 @@ libuser_prompt_setattr(struct libuser_prompt *self, const char *attr,
 			DEBUG_EXIT;
 			return -1;
 		}
-		g_free((char *) self->prompt.domain);
-		self->prompt.domain = g_strdup(PyString_AsString(args));
+		g_free((char *)me->prompt.domain);
+		me->prompt.domain = g_strdup(PyString_AsString(args));
 		DEBUG_EXIT;
 		return 0;
 	}
@@ -309,13 +323,13 @@ libuser_prompt_setattr(struct libuser_prompt *self, const char *attr,
 			DEBUG_EXIT;
 			return -1;
 		}
-		g_free((char *) self->prompt.key);
-		self->prompt.key = g_strdup(PyString_AsString(args));
+		g_free((char *)me->prompt.key);
+		me->prompt.key = g_strdup(PyString_AsString(args));
 		DEBUG_EXIT;
 		return 0;
 	}
 	if (strcmp(attr, "visible") == 0) {
-		self->prompt.visible = PyObject_IsTrue(args);
+		me->prompt.visible = PyObject_IsTrue(args);
 		DEBUG_EXIT;
 		return 0;
 	}
@@ -327,11 +341,9 @@ libuser_prompt_setattr(struct libuser_prompt *self, const char *attr,
 			DEBUG_EXIT;
 			return -1;
 		}
-		g_free((char *) self->prompt.default_value);
-		self->prompt.default_value =
-		    (args == Py_None) ?
-		    NULL :
-		    g_strdup(PyString_AsString(args));
+		g_free((char *)me->prompt.default_value);
+		me->prompt.default_value = (args == Py_None)
+			? NULL : g_strdup(PyString_AsString(args));
 		DEBUG_EXIT;
 		return 0;
 	}
@@ -342,11 +354,10 @@ libuser_prompt_setattr(struct libuser_prompt *self, const char *attr,
 			DEBUG_EXIT;
 			return -1;
 		}
-		if (self->prompt.value && self->prompt.free_value)
-			self->prompt.free_value(self->prompt.value);
-		self->prompt.value = g_strdup(PyString_AsString(args));
-		self->prompt.free_value =
-		    (typeof(self->prompt.free_value)) g_free;
+		if (me->prompt.value && me->prompt.free_value)
+			me->prompt.free_value(me->prompt.value);
+		me->prompt.value = g_strdup(PyString_AsString(args));
+		me->prompt.free_value = g_free;
 		DEBUG_EXIT;
 		return 0;
 	}
@@ -356,51 +367,51 @@ libuser_prompt_setattr(struct libuser_prompt *self, const char *attr,
 }
 
 static int
-libuser_prompt_print(struct libuser_prompt *self, FILE * fp, int flags)
+libuser_prompt_print(PyObject *self, FILE *fp, int flags)
 {
+	struct libuser_prompt *me;
+
 	(void)flags;
+	me = (struct libuser_prompt *)self;
 	fprintf(fp,
-		"(key = \"%s\", prompt = \"%s\", domain = \"%s\", visible = %s, default_value = \"%s\", value = \"%s\")",
-		self->prompt.key ? : "",
-		self->prompt.prompt ? : "",
-		self->prompt.domain ? : "",
-		self->prompt.visible ? "true" : "false",
-		self->prompt.default_value ? : "",
-		self->prompt.value ? : "");
+		"(key = \"%s\", prompt = \"%s\", domain = \"%s\", "
+		"visible = %s, default_value = \"%s\", value = \"%s\")",
+		me->prompt.key ? me->prompt.key : "",
+		me->prompt.prompt ? me->prompt.prompt : "",
+		me->prompt.domain ? me->prompt.domain : "",
+		me->prompt.visible ? "true" : "false",
+		me->prompt.default_value ? me->prompt.default_value : "",
+		me->prompt.value ? me->prompt.value : "");
 	return 0;
 }
 
-static struct libuser_prompt *
-libuser_prompt_new(void)
+static PyObject *
+libuser_prompt_new(PyObject *ignored_self, PyObject *ignore)
 {
 	struct libuser_prompt *ret;
+
+	(void)ignored_self;
+	(void)ignore;
 	DEBUG_ENTRY;
 	ret = PyObject_NEW(struct libuser_prompt, &PromptType);
 	if (ret != NULL) {
 		memset(&ret->prompt, 0, sizeof(ret->prompt));
 	}
 	DEBUG_EXIT;
-	return ret;
+	return (PyObject *)ret;
 }
 
 static PyTypeObject PromptType = {
 	PyObject_HEAD_INIT(&PyType_Type)
-	    0,
-	"Prompt",
-	sizeof(struct libuser_prompt),
 	0,
+	"Prompt",		/* tp_name */
+	sizeof(struct libuser_prompt), /* tp_basicsize */
+	0,			/* tp_itemsize */
 
-	(destructor) libuser_prompt_destroy,
-	(printfunc) libuser_prompt_print,
-	(getattrfunc) libuser_prompt_getattr,
-	(setattrfunc) libuser_prompt_setattr,
-	(cmpfunc) NULL,
-	(reprfunc) NULL,
-
-	(PyNumberMethods *) NULL,
-	(PySequenceMethods *) NULL,
-	(PyMappingMethods *) NULL,
-	(hashfunc) NULL,
-	(ternaryfunc) NULL,
-	(reprfunc) NULL,
+	libuser_prompt_destroy,	/* tp_dealloc */
+	libuser_prompt_print,	/* tp_print */
+	libuser_prompt_getattr,	/* tp_getattr */
+	libuser_prompt_setattr,	/* tp_setattr */
+	NULL,			/* tp_compare */
+	NULL,			/* tp_repr */
 };
